import java.rmi.*;
import net.jini.core.discovery.*;
import net.jini.discovery.*;
import net.jini.core.lookup.*;
import net.jini.core.lease.*;
import net.jini.lease.*;
import java.io.*;
import media.common.*;
import java.net.*;

public class ServerPart implements DiscoveryListener, LeaseListener{
    //Variable Declaration
    protected LeaseRenewalManager leaseManager;
    private WhoIsHost host;
    private String localhost;
    
    /**
     *  This is a simple service registration class
     *  that registers the services with all available
     *  lookup servers.
     */

    public static void main(String[ ] args){

	new ServerPart();
	/**
	 * keep the server running forever to allow
	 * time for locator discovery and
	 * keep reregistering the lease
	 */
	Object keepAlive = new Object( );
	synchronized(keepAlive){
	    try{
		keepAlive.wait( );
	    }catch(InterruptedException e){
		System.out.println("Oops: the server could not sleep");
	    }
	}//end synchronized
    }

    /**
     *  instantiates the registration 
     *  process and creates the service
     */
    public ServerPart(){
 	System.setSecurityManager(new RMISecurityManager( ));
	leaseManager = new LeaseRenewalManager();
	try{
	    localhost = InetAddress.getLocalHost().getHostName( );
	}catch(Exception e){
	    System.out.println("Could not get localhost");
	}

	String nameOfHost = System.getProperty("user.name");
	if (nameOfHost == null)
	    nameOfHost = System.getProperty("log.name");
	nameOfHost = nameOfHost + "@" + localhost;
	System.out.println("You are ["+nameOfHost+"]");
	host = new WhoIsHost(localhost,nameOfHost);
	LookupDiscovery discover = null;

	try{
	    discover = new LookupDiscovery(LookupDiscovery.ALL_GROUPS);
	}catch(Exception e){
	    System.err.println(e.toString( ));
	    System.out.println("Could not create the discover in the server");
	}
	discover.addDiscoveryListener(this);

    }

    public synchronized void discovered(DiscoveryEvent dev){
	//get the lookup servers currently available

	ServiceRegistrar[] registrars = dev.getRegistrars( );
	//go through each one of the loopup servers discovered and 
	//register your service

	try{
	    for(int i=0; i<registrars.length; i++){
		ServiceRegistrar registrar = registrars[i];
		System.out.println("Registrar found at "+registrar.getLocator( ).getHost( ));
		ServiceItem item = new ServiceItem(null,new VideoConferenceImpl(host),null);
		// lease the service for 5 minutes but the 
		//actual lease time is the lookupserver's call
		ServiceRegistration reg = null;
		long actuallease = 0;
		try{
		    reg = registrar.register(item, 300000L);
		    actuallease = (reg.getLease().getExpiration() -
				   System.currentTimeMillis()) / 1000;	
		}catch(RemoteException e){
		    System.err.println("Registration exception: "+e.toString( ));
		}
	    
		System.out.println("Service registered, expires in " +
				   actuallease + " seconds.");
		leaseManager.renewUntil(reg.getLease( ), Lease.FOREVER,this);
	    }//end for
	}catch(Exception e){
	    System.out.println("error");
	}
    }//end method

    public synchronized void discarded(DiscoveryEvent dev){
    }
    
    public void notify(LeaseRenewalEvent e){
	System.out.println("Lease Expired "+e.toString( ));
    }
}//end of class
    
// 	if(nameOfHost.equalsIgnoreCase("none")){
// 	    host = new WhoIsHost( );
// 	    video = new VideoConferenceImpl(host);
// 	}
// 	else{
// 	    host = new WhoIsHost(nameOfHost);
// 	    video = new VideoConferenceImpl(host);
// 	}
// 	LookupLocator locator = null;
// 	try {
// 	    //expect a lookupservice on the local machine
// 	    locator = new LookupLocator("jini://localhost");
// 	    ServiceRegistrar registrar = locator.getRegistrar( );
// 	    System.out.println("Registrar found, location = "+registrar.getLocator( ).getHost( ));
// 	    ServiceItem item = new ServiceItem(null, video, null);
// 	       // the lookup server's call

// 	}catch (Exception e) {
// 	    System.out.println("error");
// 	}
//     }
 







