
package media.config;

import java.io.*;

/**
 *
 * Title:        Schemes.java<br>
 * Description:  Allows diffrent schemes to be read in and processed.<br>
 * Copyright:    Konrad Rzeszutek<br>
 * Company:      darnok.dhs.org<p>
 *
 * Version:     $Id: Schemes.java,v 1.6 2000/11/16 05:53:50 konrad Exp $ <br>
 *<br><br>
 * Allows diffrent Scheme objects to be retrieved.
 */


public class Schemes {

  // Our init file.
  private static String SCHEME_INIT_FILE    = "/scheme.ini";

  /**
   * Init the object.
   *
   * <br><b>Parameters</b>:<br>
   *   scheme_directory should be the directory under which the
   *   schemes are located.<br>
   *   config should be the Config object which has been hopefully initialized
   *   prior to calling this class. Can be null.
   * <br>
   * <i>FileNotFoundException</i> is thrown when one of the init file in the
   * scheme directory points to a non-existing file.<br>
   * <i>StreamCorruptedException</i> is thrown when the format of the init
   * file doesn't match the format.
   */
  public Schemes(String scheme_directory,Config config) throws FileNotFoundException,
                                                 StreamCorruptedException {

    this.config = config; // That's b/c we want to use the 'write' method.

    File scheme_root_directory = new File(scheme_directory);
    if (!scheme_root_directory.exists())
      throw new FileNotFoundException("Couldn't find the "+ scheme_directory+" directory!");

    // scan the directories and find those that have 'scheme.ini' file
    File[] file_schemes = scanDirectory(scheme_root_directory);

    if ((file_schemes == null) || (file_schemes.length < 1))
      throw new FileNotFoundException("Couldn't find any schemes in "+ scheme_directory+"!");
    // read them in

    schemes = new Scheme[file_schemes.length];

    for (int loop0 = 0; loop0 < file_schemes.length;loop0 ++)
      schemes[loop0] = readScheme(file_schemes[loop0]);

  }

  /**
   * Scans the directory for scheme directories.
   *
   *
   * <br><b>Parameters</b>:<br>
   *   != null && scheme_root_directory.exists() && scheme_root_directory.isDirectory()
   *
   * <br><b>return:</b><br>
   *  null if scheme_root_directory.isDirectory() == false<br>
   *  File[] if some (more than zero) subdirectories of scheme_root_directory
   *  have the 'scheme.ini' file.
   */
  private File[] scanDirectory(File scheme_root_directory) {

    if (!scheme_root_directory.isDirectory())
      return null;

    File[] temp_list = scheme_root_directory.listFiles();

    int directories_length = 0;

    // Look thru the directories and try to find in which ones the
    // 'scheme.ini' file is located.
    // This way we can find out the length of the array we are
    // returning.
    for (int loop0 = 0; loop0 < temp_list.length; loop0++)
      if (temp_list[loop0].isDirectory())
        if (new File(temp_list[loop0].getAbsolutePath()+SCHEME_INIT_FILE).exists()) {
          directories_length++;

        }

    File[] directories = new File[directories_length];

    // Dumb repetition, but what else can I do?
    // This one actually puts the names of the directories into
    // 'directories' variable.
    int loop1 = 0;
    for (int loop0 = 0; loop0 < temp_list.length; loop0++)
      if (temp_list[loop0].isDirectory())
        if (new File(temp_list[loop0].getAbsolutePath()+SCHEME_INIT_FILE).exists()) {
          directories[loop1] = temp_list[loop0];
          loop1++;

      }

    return directories;
  }

  /**
   * Reads the scheme init file and creates a object out of it.
   *
   * <br><b>Parameters</b>:<br>
   *  File != null
   * <br><b>return:</b><br>
   *  Scheme != null || throws StreamCorruptedException ||
   *  throw FileNotFoundException
   */
  private Scheme readScheme(File scheme_directory) throws StreamCorruptedException,
                                                          FileNotFoundException
                                                            {

    write("Reading "+scheme_directory.getName());
    File scheme_init_file = new File(scheme_directory + SCHEME_INIT_FILE);

    // Just making sure.
    if (!scheme_init_file.exists() && scheme_init_file.canRead())
      return null;

    // Open da bitch up!
    StreamTokenizer st;
    BufferedReader bf;
    try {
      bf = new BufferedReader(
                               new FileReader(scheme_init_file));
      st = new StreamTokenizer(bf);


    } catch (Exception e) { return null; }

    String[] scheme_string = new String[match_str.length];
    int[] scheme_ints = new int[match_ints.length];

    String manipulate = "";

    // Look for matches
    try {

      while (st.nextToken() != StreamTokenizer.TT_EOF) {

        if (st.ttype == StreamTokenizer.TT_WORD) {
          manipulate = st.sval.toLowerCase();

        // Check for file-name types
        for (int loop0 = 0; loop0 < match_str.length; loop0++)
          if (match_str[loop0].compareTo(manipulate)==0) {// match found
              st.nextToken();
              if (loop0 == 0)
                scheme_string[loop0] = st.sval;
              else
                scheme_string[loop0] = scheme_directory.getAbsolutePath()+"/" + st.sval;
              if (!new File(scheme_string[loop0]).exists() && (loop0!=0))
                 throw new FileNotFoundException("Couldn't find "+scheme_string[loop0]+"!");
              loop0 = match_str.length;
          }
        // Check for integer type.
        for (int loop0 = 0;loop0 < match_ints.length; loop0++)
          if (match_ints[loop0].compareTo(manipulate)==0) {
              st.nextToken();
              scheme_ints[loop0]=(int)st.nval;
              st.nextToken();
              scheme_ints[loop0+1]=(int)st.nval;
              loop0 = match_ints.length;
          }
        }
      }
      bf.close();
    } catch (Exception e) {
      if (e instanceof FileNotFoundException)
        throw new FileNotFoundException(e.getMessage());
      else
        throw new StreamCorruptedException("Wrong format in "+scheme_init_file+" or one of the files is non-existent");
    }

    // Create a new Scheme.
    write("Read ["+scheme_string[0]+"] in");
    return new Scheme(
                      scheme_string[0], scheme_string[1], scheme_string[2],
                      scheme_string[3], scheme_string[4], scheme_string[5],
                      scheme_string[6], scheme_string[7], scheme_string[8],
                      scheme_string[9], scheme_string[10],scheme_string[11],
                      scheme_ints[0], scheme_ints[1], scheme_ints[2],
                      scheme_ints[3], scheme_ints[4], scheme_ints[5],
                      scheme_ints[6], scheme_ints[7], scheme_ints[8],
                      scheme_ints[9], scheme_ints[10], scheme_ints[11],
                      scheme_ints[12],scheme_ints[13], scheme_ints[14],
                      scheme_ints[15]);

    }


  /**
   * Get a description list of the schemes.
   *
   * <br><b>return:</b><br>
   *   null || getSchemeNames.length() > 0
   */

   public String[] getSchemeNames() {

    String[] names = new String[schemes.length];

    for (int loop0 = 0; loop0 < schemes.length; loop0++)
      names[loop0] = schemes[loop0].name;

    return names;
  }

  /**
   * Retrieve the specific scheme which matches the given
   * string. The string has to be retrieved from <i>getSchemeNames</i>
   *
   * <br><b>Parameters</b>:<br>
   *   scheme - a string which has to be retrieved from <i>getSchemeNames</i>
   *<br>
   * <br><b>return:</b><br>
   *   Scheme
   */
  public Scheme getScheme(String scheme) {

    for (int loop0 = 0; loop0 < schemes.length; loop0 ++)
      if (schemes[loop0].name.compareTo(scheme) == 0)
        return schemes[loop0];

    // Shouldn't happen, but just in case the programmer is a bozo.
    return null;
  }

  /**
   * Writes out diagnostic informations. If Config is not initialized
   * in the constructor, this method will still work.
   */
  public void write(String s) {

    if (config != null)
      config.write(s);
  }

  // Our format for the 'scheme.ini' file - the one with files.

  private String[]  match_str     = {"name",
                                     "configureimage",
                                     "backgroundimage",
                                     "connectimage",
                                     "disconnectimage",
                                     "setupimage",
                                     "quitimage",
                                     "busyimage",
                                     "availableimage",
                                     "naimage",
                                     "connectokimage",
                                     "connectcancelimage"};

  // Our format for the 'scheme.ini' file - the ones with "x,y" position.

  private String[] match_ints        =
                                    {"connectlocation",
                                     "",
                                     "disconnectlocation",
                                     "",
                                     "setuplocation",
                                     "",
                                     "quitlocation",
                                     "",
                                     "playerlocation",
                                     "",
                                     "statuslocation",
                                     "",
                                     "connectoklocation",
                                     "",
                                     "connectcancellocation",
                                     ""};
  private Scheme[] schemes;
  private Config config;
}