package media.network;


import javax.swing.*;
import javax.media.*;
import java.net.*;
import java.awt.*;
import java.awt.event.*;
import javax.media.format.*;
import javax.media.control.*;
import javax.media.protocol.*;

import javax.media.rtp.rtcp.*;
import javax.media.rtp.event.*;
import javax.media.datasink.*;
import javax.media.rtp.*;
import javax.media.rtp.SessionManager;

import media.gui.*;
import media.config.*;

/**
 *
 * Title:        AudioReceiver.java<br>
 * Copyright:    Konrad Rzeszutek <br>
 * Company:      darnok.dhs.org<br>
 *
 * Version:     $Id: VideoReceiver.java,v 1.3 2000/11/28 03:41:28 konrad Exp $ <br>
 *
 * Receivers the video stream using RTP. This class
 * operates (ie gets its date) from config.Config
 */
public class VideoReceiver implements ControllerListener,
                                      ReceiveStreamListener {

  public Player video_player   = null;
  private Component visualComponent = null;

  private OcularBase gui        = null;
  private Config config         = null;
  private SessionManager m      = null;
  private boolean videoPlayerRealized = false; //boolean if video player realized
  public final static int OK    = 94321;
  public final static int ERROR = 128321;
  /**
   * Initalizes the VideoManager with the OcularBase and Config objects.
   * <br><b>Parameters</b>:<br>
   *   gui != null<br>
   *   config != null
   */

  public VideoReceiver(OcularBase gui, Config config) {

      this.gui = gui;
      this.config = config;
      config.write("Video receiver started");
  }

  /**
    * Starts the receiver. Without starting this method, no
    * data will be received at all.
    */
    public synchronized int startVideoReceiver() {

      if (resetVideo() == OK)
        m = createVideoManager();
      else
        m = null;

      if (m == null)
        return ERROR;

      return OK;
    }

    /**
     * Resets RTP incoming video receiver to its initial state.
     */
    public synchronized int resetVideo() {

      int return_state = OK;

      if (video_player == null) {
	  if (m != null) {
	      m.removeReceiveStreamListener(this);
	      m.removeAllPeers();
	      m.closeSession("bye!");
	  }
	  return return_state;
      }
      try {
        video_player.stop();
        video_player.deallocate();
        videoPlayerRealized = false;
      } catch (Exception e) {
        config.write(e.getMessage());
        return_state = ERROR;
      }
      video_player = null;
      if (m != null) {
        m.removeAllPeers();
        m.closeSession("I want to close");
      }

      return return_state;
  }

  /**
    * Starts the receiver. Without starting this method, no
    * data will be received at all.
  */

  private SessionManager createVideoManager () {


      SessionManager mgr = (SessionManager)new com.sun.media.rtp.RTPSessionMgr();

      if (mgr == null) return null;

    /* Go thru the format and change it */
      mgr.addFormat(new VideoFormat(config.DEFAULT_VIDEO_ENC),1);

    /* update(ReceiveStreamEvent) handles incoming stuff */
    mgr.addReceiveStreamListener(this);

    SessionAddress localaddr = new SessionAddress();

    try{
       InetAddress destaddr = InetAddress.getByName(config.RECV_HOSTNAME);
       config.write("Video receiver is :"+destaddr);
       mgr.initSession(localaddr,
                         null,
                         0.05,
                         0.25);
       SessionAddress sessaddr = new SessionAddress(destaddr,
                                                    config.RECV_VID_PORT,
                                                    destaddr,
                                                    config.RECV_VID_PORT + 1);
       /* ttl is only if outgoing address is multicast */
       mgr.startSession(sessaddr,config.TTL,null);

    } catch (Exception e) {
       config.write("Couldn't initialize Video RTP");
       return null;
   }

   return mgr;

  }

  /*
   * This dude is awaken whenever a incoming stream is comming.
   *
   */

 public void update( ReceiveStreamEvent event) {

   // find the sourceRTPSM for this event
   SessionManager source = (SessionManager)event.getSource();
   //gui.write("V_RTP:"+source);
   // create a new player if a new recvstream is detected
   if (event instanceof NewReceiveStreamEvent) {

       ReceiveStream stream = null;
       String cname = null;
       try {
            // get a handle over the ReceiveStream
            stream =((NewReceiveStreamEvent)event)
                               .getReceiveStream();

            Participant part = stream.getParticipant();

            if (part != null) cname = part.getCNAME();
            gui.write("Incoming VideoRTP session from: "+cname);
            // get a handle over the ReceiveStream datasource
            DataSource dsource = stream.getDataSource();

            // create a player by passing datasource to the
            // Media Manager
            video_player = javax.media.Manager.createPlayer(dsource);

       } catch (Exception e) {
            System.err.println("NewReceiveStreamEvent exception "
                                          + e.getMessage());
            return;
       }

       if (video_player == null) return;

       video_player.addControllerListener(this);
       video_player.realize();
   }
  }


   /**
    * This controllerUpdate function must be defined in order
    * to implement a ControllerListener interface.  This
    * function will be called whenever there is a media event.
    */
   public synchronized void controllerUpdate(ControllerEvent event) {

       //gui.write("V_RTP:"+event);

      if (video_player == null) return;

      // When the player is Realized, get the visual
      // component and add to the GUI.

      if (event instanceof RealizeCompleteEvent)
      {
          if (video_player.getState() == Controller.Realized)
            if ((visualComponent = video_player.getVisualComponent()) != null)  {

                   gui.setupConnectedVideo(visualComponent);
                   video_player.start();
                   videoPlayerRealized = true;
            }
        }
    }

    /**
     * Returns if video player is realized
     */
    public boolean isRealized(){
	return videoPlayerRealized; 
    }
 

}




